import 'dart:async';
import 'dart:io' as which_platform;
import 'package:flutter/foundation.dart';
import 'package:device_info_plus/device_info_plus.dart';
import 'package:geolocator/geolocator.dart';
import 'package:geocoding/geocoding.dart';
import 'package:nuntio_hera/hera/hera.dart';
import 'package:url_launcher/url_launcher.dart';

extension HeraExtenstion on Hera {
  static final DeviceInfoPlugin _deviceInfoPlugin = DeviceInfoPlugin();

  static Future<void> redirect(Uri authorizationUrl) async {
    if (await canLaunchUrl(authorizationUrl)) {
      await launchUrl(authorizationUrl, mode: LaunchMode.inAppWebView);
    }
  }

  /// Determine the name of the device.
  static Future<String> getDeviceInfo() async {
    try {
      if (kIsWeb) {
        return "Web";
      } else {
        if (which_platform.Platform.isAndroid) {
          return (await _deviceInfoPlugin.androidInfo).host ?? "";
        } else if (which_platform.Platform.isIOS) {
          return (await _deviceInfoPlugin.iosInfo).name ?? "";
        } else if (which_platform.Platform.isLinux) {
          return (await _deviceInfoPlugin.linuxInfo).name;
        } else if (which_platform.Platform.isMacOS) {
          return (await _deviceInfoPlugin.macOsInfo).computerName;
        } else if (which_platform.Platform.isWindows) {
          return (await _deviceInfoPlugin.windowsInfo).computerName;
        }
      }
    } catch (e) {
      print("could not get device info with err: " + e.toString());
    }
    return "";
  }

  /// Determine the current position of the device.
  /// When the location services are not enabled or permissions
  /// are denied the `Future` will return an error.
  static Future<Placemark?> determinePlacemark() async {
    if (kIsWeb) {
      return null;
    }
    try {
      bool serviceEnabled;
      LocationPermission permission;

      // Test if location services are enabled.
      serviceEnabled = await Geolocator.isLocationServiceEnabled();
      if (!serviceEnabled) {
        // Location services are not enabled don't continue
        // accessing the position and request users of the
        // App to enable the location services.
        print(
            'Location services is disabled. Enable it to see where your users log in from.');
        return null;
      }

      permission = await Geolocator.checkPermission();
      if (permission == LocationPermission.unableToDetermine) {
        return null;
      } else if (permission == LocationPermission.denied) {
        permission = await Geolocator.requestPermission();
        if (permission == LocationPermission.denied) {
          // Permissions are denied, next time you could try
          // requesting permissions again (this is also where
          // Android's shouldShowRequestPermissionRationale
          // returned true. According to Android guidelines
          // your App should show an explanatory UI now.
          return Future.error('Location permissions are denied');
        }
      }
      if (permission == LocationPermission.deniedForever) {
        // Permissions are denied forever, handle appropriately.
        return null;
      }
      // When we reach here, permissions are granted and we can
      // continue accessing the position of the device.
      Position position = await Geolocator.getCurrentPosition();
      List<Placemark> placemarks =
          await placemarkFromCoordinates(position.latitude, position.longitude);
      return placemarks.first;
    } catch (e) {
      print("could not get location with err: " + e.toString());
      return null;
    }
  }
}
